<?php

namespace GeorgGriesser\WordPress\Velocious\Smart_WP\Header_Manager;

use GeorgGriesser\WordPress\Velocious\Smart_WP\Header_Generator;

class Object_Header_Manager extends Header_Manager
{
    protected $supported_post_types_filter_name = 'smart_wp_supported_post_types_object';

    /**
     * Sets headers on post object pages (posts, pages, attachments, custom
     * post types).
     *
     * In order to calculate the modified time, two time sources are used:
     *   1) the post object's modified time.
     *   2) the modified time of the most recent comment that is attached to the post object.
     *
     * The most "recent" timestamp of the two is used.
     */
    function set_headers($post)
    {
        if (post_password_required()) {
            return;
        }

        if (!$this->is_valid_post_type($post)) {
            return;
        }

        $post_modified_time = $post->post_modified_gmt;
        $modified_time = strtotime($post_modified_time);

        $comment_modified_time = $this->get_comment_modified_time($post);

        if ($comment_modified_time > $modified_time) {
            $modified_time = $comment_modified_time;
        }

        $this->headers = Header_Generator::batch_generate_headers($post, $modified_time, $this->options);
    }

    /**
     * @param \WP_Post $post
     *
     * @return int
     */
    private function get_comment_modified_time($post)
    {
        $comments = get_comments(array(
            'status' => 'approve',
            'orderby' => 'comment_date_gmt',
            'number' => '1',
            'post_id' => $post->ID
        ));

        if (!empty($comments)) {
            $comment = $comments[0];

            $comment_modified_time = strtotime($comment->comment_date_gmt);

            return $comment_modified_time;
        }

        return strtotime(0);
    }
}