<?php

namespace GeorgGriesser\WordPress\Velocious\Smart_WP;


class Settings_Page
{

    public function init()
    {
        add_action('admin_menu', array($this, 'add_page'));

        add_action('admin_init', array($this, 'settings_init'));

    }

    public function add_page()
    {
        $page_hook = add_options_page('Smart WordPress', 'Smart WordPress', 'manage_options', 'smart-wp', array($this, 'render'));

        add_action('load-' . $page_hook, array($this, 'add_remaining_callbacks'));
    }

    public function add_remaining_callbacks()
    {
        add_action('admin_enqueue_scripts', array($this, 'add_styles'));
    }

    public function add_styles()
    {
        wp_enqueue_style('fast-wp', get_smart_wp_plugin_style_url('styles.css'));
    }


    function settings_init()
    {
        // register a new setting for "smart_wp" page
        register_setting('smart_wp_etag_group', 'smart_wp_settings', array($this, 'sanitize_input'));
        register_setting('smart_wp_last_modified_group', 'smart_wp_settings', array($this, 'sanitize_input'));
        register_setting('smart_wp_cache_control_group', 'smart_wp_settings', array($this, 'sanitize_input'));
        register_setting('smart_wp_pre_existing_headers_group', 'smart_wp_settings', array($this, 'sanitize_input'));


        // register a new section in the "smart_wp" page
        add_settings_section(
            'smart_wp_section_etag',
            false,
            array($this, 'render_section_intro'),
            'smart_wp_etag'
        );

        // register a new section in the "smart_wp" page
        add_settings_section(
            'smart_wp_section_last_modified',
            false,
            array($this, 'render_section_intro'),
            'smart_wp_last_modified'
        );

        // register a new section in the "smart_wp" page
        add_settings_section(
            'smart_wp_section_cache_control',
            false,
            array($this, 'render_section_intro'),
            'smart_wp_cache_control'
        );

        // register a new section in the "smart_wp" page
        add_settings_section(
            'smart_wp_section_pre_existing_headers',
            false,
            array($this, 'render_section_intro'),
            'smart_wp_pre_existing_headers'
        );

        $this->add_settings_fields();

    }

    private function add_settings_fields()
    {
        $options = get_option('smart_wp_settings');

        $option = Options_Manager::get_option_configuration();

        foreach ($option as $key => $args) {

            add_settings_field(
                $key . '_field',
                $args['description'],
                array($this, 'field_render'),
                $args['setting'],
                $args['section'],
                [
                    'option_name' => $key,
                    'option_value' => $options[$key],
                    'field_type' => $args['field_type'],
                    'description_extended' => $args['description_extended']
                ]
            );
        }
    }

    function field_render($args)
    {
        if ($args['field_type'] === 'checkbox') {
            $this->checkbox_field_render($args);
        } else if ($args['field_type'] === 'text') {
            $this->text_field_render($args);
        }
    }

    private function text_field_render($args)
    {
        printf(
            "
                <input type='text' name='smart_wp_settings[%s]' value='%s'>
                <p>%s</p>
            ",
            $args['option_name'],
            $args['option_value'],
            $args['description_extended']
        );
    }

    private function checkbox_field_render($args)
    {
        printf(
            "
                <input type='checkbox' name='smart_wp_settings[%s]' %s value='%s'>
                <p>%s</p>
            ",
            $args['option_name'],
            checked($args['option_value'], 1, false),
            true,
            $args['description_extended']
        );
    }


    public function render()
    {
        if (!current_user_can('manage_options')) {
            return;
        }




        ?>
        <div class="wrap velocious">

            <div class="velocious-header">

                <img src="<?php echo get_smart_wp_plugin_img_url('icon-256x256.png') ?>"/>


                <p class="h1">Smart WordPress by Velocious.io</p>


            </div>

            <div class="velocious-content">

                <div class="velocious-main">

                    <h2></h2>

                    <p>
                        Thanks for using Smart WordPress by Velocious.io - <strong>you are awesome</strong>!
                    </p>
                    <p>
                        If you happen to encounter any difficulties or would like to provide me with additional
                        feedback, do not hesitate to contact me. You can either reach me in WordPress.org's support
                        forum or, if you do not have a WP.org
                        account, you can drop me a message using <a href="https://www.velocious.io/contact/">my contact
                            form</a>.
                    </p>

                    <form action="options.php" method="post">

                        <?php String_Manager::print_intro('etag'); ?>

                        <div class="velocious-main__table-wrapper">

                            <?php
                            // output security fields for the registered setting "smart_wp"
                            settings_fields('smart_wp_etag_group');
                            // output setting sections and their fields
                            // (sections are registered for "smart_wp", each field is registered to a specific section)
                            do_settings_sections('smart_wp_etag');
                            ?>

                        </div>

                        <?php String_Manager::print_intro('last-modified'); ?>

                        <div class="velocious-main__table-wrapper">

                            <?php
                            // output security fields for the registered setting "smart_wp"
                            settings_fields('smart_wp_last_modified_group');
                            // output setting sections and their fields
                            // (sections are registered for "smart_wp", each field is registered to a specific section)
                            do_settings_sections('smart_wp_last_modified');
                            ?>

                        </div>

                        <?php String_Manager::print_intro('cache-control'); ?>

                        <div class="velocious-main__table-wrapper">

                            <?php
                            // output security fields for the registered setting "smart_wp"
                            settings_fields('smart_wp_cache_control_group');
                            // output setting sections and their fields
                            // (sections are registered for "smart_wp", each field is registered to a specific section)
                            do_settings_sections('smart_wp_cache_control');
                            ?>

                        </div>

                        <?php String_Manager::print_intro('pre-existing-headers'); ?>

                        <div class="velocious-main__table-wrapper">

                            <?php
                            // output security fields for the registered setting "smart_wp"
                            settings_fields('smart_wp_pre_existing_headers_group');
                            // output setting sections and their fields
                            // (sections are registered for "smart_wp", each field is registered to a specific section)
                            do_settings_sections('smart_wp_pre_existing_headers');
                            ?>

                        </div>

                        <?php
                        // output save settings button
                        submit_button('Save Settings');
                        ?>

                    </form>
                </div>
            </div>
        </div>
        <?php
    }


    function render_section_intro($args)
    {
        // I do not like to display a section intro this way
    }


    public function sanitize_input($input)
    {
        $output = array();

        foreach ($input as $key => $value) {

            if (!isset($input[$key])) {
                $output[$key] = 0;
                continue;
            }

            if (!filter_var($value, FILTER_VALIDATE_INT)) {
                $output[$key] = 0;
                continue;
            }

            $output[$key] = $value;
        }

        return $output;
    }
}